@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2016 team free-astro (see more in AUTHORS file)
  * Reference site is http://free-astro.vinvin.tf/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -25,6 +25,7 @@
 #include <float.h>
 
 #include "core/siril.h"
+#include "core/processing.h"
 #include "gui/callbacks.h"
 #include "io/single_image.h"
 #include "gui/histogram.h"
@@ -45,6 +46,8 @@ void hsl_to_rgb(double h, double sl, double l, double * r, double * g,
 		double * b) {
 	double v;
 
+	assert(h >= 0.0 && h <= 1.0);
+	if (h >= 1.0) h -= 1.0;		// this code doesn't work for h = 1
 	v = (l <= 0.5) ? (l * (1.0 + sl)) : (l + sl - l * sl);
 	if (v <= 0) {
 		*r = *g = *b = 0.0;
@@ -141,10 +144,7 @@ void rgb_to_hsl(double r, double g, double b, double *h, double *s, double *l) {
 	*h /= 6;
 }
 
-/* In these functions h =0...360, s=0...1, v=0....1 
- * So be careful: it is not the same behaviour than rgb_to_hsl
- * and its reversal. But I think that it is a better choice */
-
+/* all variables are between 0 and 1. h takes 0 for grey */
 void rgb_to_hsv(double r, double g, double b, double *h, double *s, double *v) {
 	double cmax, cmin, delta;
 
@@ -153,29 +153,34 @@ void rgb_to_hsv(double r, double g, double b, double *h, double *s, double *v) {
 	cmin = min(r, g);
 	cmin = min(cmin, b);
 	delta = cmax - cmin;
-	*s = (delta == 0.0 ? 0.0 : delta / cmax);
 	*v = cmax;
+	if (delta == 0.0) {
+		*s = 0.0;
+		*h = 0.0;
+		return;
+	}
+	*s = delta / cmax;
 
 	if (cmax == r)
-		*h = (((g - b) / delta)) * 60.0;
+		*h = (((g - b) / delta)) / 6.0;
 	else if (cmax == g)
-		*h = (((b - r) / delta) + 2.0) * 60.0;
+		*h = (((b - r) / delta) + 2.0) / 6.0;
 	else
-		*h = (((r - g) / delta) + 4.0) * 60.0;
+		*h = (((r - g) / delta) + 4.0) / 6.0;
 
 	if (*h < 0.0)
-		*h += 360.0;
+		*h += 1.0;
 }
 
 void hsv_to_rgb(double h, double s, double v, double *r, double *g, double *b) {
 	double p, q, t, f;
 	int i;
 
-	if (h >= 360.0)
-		h -= 360.0;
-	h /= 60.0;
-	i = (int) h;
-	f = h - i;
+	if (h >= 1.0)
+		h -= 1.0;
+	h *= 6.0;
+	i = (int)h;
+	f = h - (double)i;
 	p = v * (1.0 - s);
 	q = v * (1.0 - (s * f));
 	t = v * (1.0 - (s * (1.0 - f)));
@@ -216,18 +221,9 @@ void hsv_to_rgb(double h, double s, double v, double *r, double *g, double *b) {
 }
 
 void rgb_to_xyz(double r, double g, double b, double *x, double *y, double *z) {
-	if (r > 0.04045)
-		r = pow(((r + 0.055) / 1.055), 2.4);
-	else
-		r = r / 12.92;
-	if (g > 0.04045)
-		g = pow(((g + 0.055) / 1.055), 2.4);
-	else
-		g = g / 12.92;
-	if (b > 0.04045)
-		b = pow(((b + 0.055) / 1.055), 2.4);
-	else
-		b = b / 12.92;
+	r = (r <= 0.04045) ? r / 12.92 : pow(((r + 0.055) / 1.055), 2.4);
+	g = (g <= 0.04045) ? g / 12.92 : pow(((g + 0.055) / 1.055), 2.4);
+	b = (b <= 0.04045) ? b / 12.92 : pow(((b + 0.055) / 1.055), 2.4);
 
 	r *= 100;
 	g *= 100;
@@ -243,68 +239,46 @@ void xyz_to_LAB(double x, double y, double z, double *L, double *a, double *b) {
 	y /= 100.000;
 	z /= 108.883;
 
-	if (x > 0.008856452)
-		x = pow(x, 1 / 3.);
-	else
-		x = (7.787037037 * x) + (16. / 116.);
-	if (y > 0.008856452)
-		y = pow(y, 1 / 3.);
-	else
-		y = (7.787037037 * y) + (16. / 116.);
-	if (z > 0.008856452)
-		z = pow(z, 1 / 3.);
-	else
-		z = (7.787037037 * z) + (16. / 116.);
+	x = (x > 0.008856452) ? pow(x, 1 / 3.0) : (7.787037037 * x) + (16. / 116.);
+	y = (y > 0.008856452) ? pow(y, 1 / 3.0) : (7.787037037 * y) + (16. / 116.);
+	z = (z > 0.008856452) ? pow(z, 1 / 3.0) : (7.787037037 * z) + (16. / 116.);
 
-	*L = (116 * y) - 16;
-	*a = 500 * (x - y);
-	*b = 200 * (y - z);
+	*L = (116.0 * y) - 16.0;
+	*a = 500.0 * (x - y);
+	*b = 200.0 * (y - z);
 }
 
 void LAB_to_xyz(double L, double a, double b, double *x, double *y, double *z) {
-	*y = (L + 16.) / 116.;
-	*x = a / 500. + (*y);
-	*z = *y - b / 200.;
+	*y = (L + 16.0) / 116.0;
+	*x = a / 500.0 + (*y);
+	*z = *y - b / 200.0;
 
-	if (((*x) * (*x) * (*x)) > 0.008856452)
-		*x = (*x) * (*x) * (*x);
-	else
-		*x = (*x - 16. / 116.) / 7.787037037;
-	if (((*y) * (*y) * (*y)) > 0.008856452)
-		*y = (*y) * (*y) * (*y);
-	else
-		*y = (*y - 16. / 116.) / 7.787037037;
-	if (((*z) * (*z) * (*z)) > 0.008856452)
-		*z = (*z) * (*z) * (*z);
-	else
-		*z = (*z - 16. / 116.) / 7.787037037;
+	double x3, y3, z3;
+	x3 = (*x) * (*x) * (*x);
+	y3 = (*y) * (*y) * (*y);
+	z3 = (*z) * (*z) * (*z);
 
-	*x = 95.047 * (*x);
-	*y = 100.000 * (*y);
-	*z = 108.883 * (*z);
+	*x = (x3 > 0.008856452) ? x3 : (*x - 16. / 116.) / 7.787037037;
+	*y = (y3 > 0.008856452) ? y3 : (*y - 16. / 116.) / 7.787037037;
+	*z = (z3 > 0.008856452) ? z3 : (*z - 16. / 116.) / 7.787037037;
+
+	*x *= 95.047;
+	*y *= 100.000;
+	*z *= 108.883;
 }
 
 void xyz_to_rgb(double x, double y, double z, double *r, double *g, double *b) {
-	x /= 100.;
-	y /= 100.;
-	z /= 100.;
+	x /= 100.0;
+	y /= 100.0;
+	z /= 100.0;
 
-	*r = 3.240479 * x - 1.537150 * y - 0.498535 * z;
+	*r =  3.240479 * x - 1.537150 * y - 0.498535 * z;
 	*g = -0.969256 * x + 1.875992 * y + 0.041556 * z;
-	*b = 0.055648 * x - 0.204043 * y + 1.057311 * z;
+	*b =  0.055648 * x - 0.204043 * y + 1.057311 * z;
 
-	if (*r > 0.0031308)
-		*r = 1.055 * (pow(*r, (1 / 2.4))) - 0.055;
-	else
-		*r = 12.92 * (*r);
-	if (*g > 0.0031308)
-		*g = 1.055 * (pow(*g, (1 / 2.4))) - 0.055;
-	else
-		*g = 12.92 * (*g);
-	if (*b > 0.0031308)
-		*b = 1.055 * (pow(*b, (1 / 2.4))) - 0.055;
-	else
-		*b = 12.92 * (*b);
+	*r = (*r > 0.0031308) ? 1.055 * (pow(*r, (1 / 2.4))) - 0.055 : 12.92 * (*r);
+	*g = (*g > 0.0031308) ? 1.055 * (pow(*g, (1 / 2.4))) - 0.055 : 12.92 * (*g);
+	*b = (*b > 0.0031308) ? 1.055 * (pow(*b, (1 / 2.4))) - 0.055 : 12.92 * (*b);
 }
 
 // idle function executed at the end of the extract_channels processing
@@ -372,7 +346,7 @@ gpointer extract_channels(gpointer p) {
 			double g = (double) buf[GLAYER][i] / USHRT_MAX_DOUBLE;
 			double b = (double) buf[BLAYER][i] / USHRT_MAX_DOUBLE;
 			rgb_to_hsv(r, g, b, &h, &s, &v);
-			buf[RLAYER][i] = round_to_WORD(h);// h is set between 0 and 360 in this function
+			buf[RLAYER][i] = round_to_WORD(h * 360.0);
 			buf[GLAYER][i] = round_to_WORD(s * USHRT_MAX_DOUBLE);
 			buf[BLAYER][i] = round_to_WORD(v * USHRT_MAX_DOUBLE);
 		}
@@ -440,15 +414,13 @@ gpointer enhance_saturation(gpointer p) {
 	args->h_min /= 360.0;
 	args->h_max /= 360.0;
 	if (args->preserve) {
-		double norm = (double) get_normalized_value(args->fit);
-
-		imstats *stat = statistics(args->fit, GLAYER, NULL);
+		imstats *stat = statistics(args->fit, GLAYER, NULL, STATS_BASIC);
 		bg = stat->median + stat->sigma;
-		bg /= norm;
+		bg /= stat->normValue;
 		free(stat);
 	}
 
-#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(dynamic, 1)
+#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static)
 	for (i = 0; i < args->fit->rx * args->fit->ry; i++) {
 		double h, s, l;
 		double r = (double) buf[RLAYER][i] / USHRT_MAX_DOUBLE;
@@ -511,7 +483,7 @@ gpointer scnr(gpointer p) {
 	gettimeofday(&t_start, NULL);
 
 	WORD norm = get_normalized_value(args->fit);
-#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(dynamic,1)
+#pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static)
 	for (i = 0; i < nbdata; i++) {
 		double red = (double) buf[RLAYER][i] / norm;
 		double green = (double) buf[GLAYER][i] / norm;
@@ -630,6 +602,36 @@ void initialize_calibration_interface() {
 	gtk_adjustment_set_value(selection_white_adjustment[3], 0);
 }
 
+/* This function equalize the background by giving equal value for all layers */
+static void background_neutralize(fits* fit, rectangle black_selection) {
+	int chan, i;
+	imstats** stats;
+	int ref = 0;
+
+	assert(fit->naxes[2] == 3);
+
+	stats = malloc(3 * sizeof(imstats *));
+	for (chan = 0; chan < 3; chan++) {
+		stats[chan] = statistics(fit, chan, &black_selection, STATS_BASIC);
+		ref += stats[chan]->median;
+	}
+	ref /= 3;
+
+	for (chan = 0; chan < 3; chan++) {
+		int offset = stats[chan]->mean - ref;
+		WORD *buf = fit->pdata[chan];
+		for (i = 0; i < fit->rx * fit->ry; i++) {
+			if (buf[i] < offset)
+				buf[i] = 0;
+			else
+				buf[i] = (buf[i] - offset >= USHRT_MAX ? USHRT_MAX : buf[i] - offset);
+
+		}
+		free(stats[chan]);
+	}
+	free(stats);
+}
+
 void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data) {
 	static GtkSpinButton *selection_black_value[4] = { NULL, NULL, NULL, NULL };
 	rectangle black_selection;
@@ -658,6 +660,8 @@ void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data)
 	black_selection.w = gtk_spin_button_get_value(selection_black_value[2]);
 	black_selection.h = gtk_spin_button_get_value(selection_black_value[3]);
 
+	undo_save_state("Processing: Background neutralization");
+
 	set_cursor_waiting(TRUE);
 	background_neutralize(&gfit, black_selection);
 	delete_selected_area();
@@ -667,37 +671,6 @@ void on_button_bkg_neutralization_clicked(GtkButton *button, gpointer user_data)
 	set_cursor_waiting(FALSE);
 }
 
-/* This function equalize the background by giving equal value for all layers
- * The bkg is usually of 10% of the full dynamic */
-void background_neutralize(fits* fit, rectangle black_selection) {
-	int layer, i;
-	imstats** stats;
-	int ref = 0;
-
-	stats = malloc(com.uniq->nb_layers * sizeof(imstats *));
-	for (layer = 0; layer < com.uniq->nb_layers; layer++) {
-		stats[layer] = statistics(fit, layer, &black_selection);
-		ref += stats[layer]->median;
-	}
-	ref /= 3;
-
-	for (layer = 0; layer < com.uniq->nb_layers; layer++) {
-		int offset = stats[layer]->mean - ref;
-		WORD *buf = fit->pdata[layer];
-		for (i = 0; i < fit->rx * fit->ry; i++) {
-			if (buf[i] < offset)
-				buf[i] = 0;
-			else
-				buf[i] = (
-						buf[i] - offset >= USHRT_MAX ?
-								USHRT_MAX : buf[i] - offset);
-
-		}
-		free(stats[layer]);
-	}
-	free(stats);
-}
-
 void on_button_white_selection_clicked(GtkButton *button, gpointer user_data) {
 	static GtkSpinButton *selection_white_value[4] = { NULL, NULL, NULL, NULL };
 
@@ -724,61 +697,115 @@ void on_button_white_selection_clicked(GtkButton *button, gpointer user_data) {
 	gtk_spin_button_set_value(selection_white_value[3], com.selection.h);
 }
 
-void get_coeff_for_wb(fits *fit, rectangle selection, double coef[]) {
-	int chan, i;
-	gsl_histogram *histo[3];
-	double maxi[3], minimum = DBL_MAX;
+static void get_coeff_for_wb(fits *fit, rectangle white, rectangle black,
+		double kw[], double bg[], double *norm, double low, double high) {
+	int chan, i, j, n;
+	double tmp[3] = { 0.0, 0.0, 0.0 };
+	WORD lo, hi;
 
-	assert(fit->naxes[2] == 1 || fit->naxes[2] == 3);
+	assert(fit->naxes[2] == 3);
 
-	for (chan = 0; chan < fit->naxes[2]; chan++) {
-		histo[chan] = computeHisto_Selection(fit, chan, &selection);
-		maxi[chan] = gsl_histogram_max_val(histo[chan]);
-		size_t bin = gsl_histogram_max_bin(histo[chan]);
-		int k = 1;
-		for (i = -20; i < 20; i++) {
-			if ((bin + i > USHRT_MAX) || ((int) bin + i < 0))
-				continue;
-			maxi[chan] += gsl_histogram_get(histo[chan], bin + i);
-			k++;
+	*norm = (double) get_normalized_value(fit);
+	lo = round_to_WORD(low * (*norm));
+	hi = round_to_WORD(high * (*norm));
+
+	for (chan = 0; chan < 3; chan++) {
+		n = 0;
+		WORD *from = fit->pdata[chan] + (fit->ry - white.y - white.h) * fit->rx
+				+ white.x;
+		int stridefrom = fit->rx - white.w;
+
+		for (i = 0; i < white.h; i++) {
+			for (j = 0; j < white.w; j++) {
+				if (*from > lo && *from < hi ) {
+					kw[chan] += (double) *from / *norm;
+					n++;
+				}
+				from++;
+			}
+			from += stridefrom;
 		}
-		maxi[chan] /= k;
-		minimum = min(maxi[chan], minimum);
-		gsl_histogram_free(histo[chan]);
+		if (n > 0)
+			kw[chan] /= (double) n;
+	}
+
+	siril_log_message("Background reference:\n");
+	for (chan = 0; chan < 3; chan++) {
+		imstats *stat = statistics(fit, chan, &black, STATS_BASIC);
+		bg[chan] = stat->median / stat->normValue;
+		siril_log_message("B%d : %.5e\n", chan, bg[chan]);
+		free(stat);
+	}
+
+	siril_log_message("White reference:\n");
+	for (chan = 0; chan < 3; chan++) {
+		siril_log_message("W%d : %.5e\n", chan, kw[chan]);
+		kw[chan] = fabs(kw[chan] - bg[chan]);
+	}
+
+	int rc = (kw[0] > kw[1]) ? ((kw[0] > kw[2]) ? 0 : 2) :
+					((kw[1] > kw[2]) ? 1 : 2);
+	for (chan = 0; chan < 3; chan++) {
+		if (chan == rc)
+			tmp[chan] = 1;
+		else
+			tmp[chan] = kw[rc] / kw[chan];
 	}
 
 	siril_log_message("Color calibration factors:\n");
-	for (chan = 0; chan < fit->naxes[2]; chan++) {
-		coef[chan] = maxi[chan] / minimum;
-		siril_log_message("K%d=%0.3lf\n", chan, coef[chan]);
+	for (chan = 0; chan < 3; chan++) {
+		kw[chan] = tmp[chan];
+		siril_log_message("K%d : %5.3lf\n", chan, kw[chan]);
 	}
 }
 
-void white_balance(fits *fit, gboolean is_manual, rectangle white_selection,
+static int calibrate(fits *fit, int layer, double kw, double bg, double norm) {
+	WORD *buf;
+	WORD bgNorm;
+	int i;
+
+	bgNorm = bg * norm;
+
+	buf = fit->pdata[layer];
+	for (i = 0; i < fit->rx * fit->ry; ++i) {
+		buf[i] = round_to_WORD((buf[i] - bgNorm) * kw + bgNorm);
+	}
+	return 0;
+}
+
+static void white_balance(fits *fit, gboolean is_manual, rectangle white_selection,
 		rectangle black_selection) {
-	int chan, nb_chan;
-	double coef[3];
+	int chan;
+	double norm, low, high;
+	double kw[3] = { 0.0, 0.0, 0.0 };
+	double bg[3] = { 0.0, 0.0, 0.0 };
 	static GtkRange *scale_white_balance[3] = { NULL, NULL, NULL };
+	static GtkRange *scaleLimit[2] = { NULL, NULL };
 
 	if (scale_white_balance[RLAYER] == NULL) {
 		scale_white_balance[RLAYER] = GTK_RANGE(lookup_widget("scale_r"));
 		scale_white_balance[GLAYER] = GTK_RANGE(lookup_widget("scale_g"));
 		scale_white_balance[BLAYER] = GTK_RANGE(lookup_widget("scale_b"));
+
+		scaleLimit[0] = GTK_RANGE(lookup_widget("lowWhiteColorCalibScale"));
+		scaleLimit[1] = GTK_RANGE(lookup_widget("upWhiteColorCalibScale"));
 	}
 
-	assert(fit->naxes[2] == 1 || fit->naxes[2] == 3);
-	nb_chan = fit->naxes[2];
+	assert(fit->naxes[2] == 3);
 
 	if (is_manual) {
-		coef[RLAYER] = gtk_range_get_value(scale_white_balance[RLAYER]);
-		coef[GLAYER] = gtk_range_get_value(scale_white_balance[GLAYER]);
-		coef[BLAYER] = gtk_range_get_value(scale_white_balance[BLAYER]);
+		kw[RLAYER] = gtk_range_get_value(scale_white_balance[RLAYER]);
+		kw[GLAYER] = gtk_range_get_value(scale_white_balance[GLAYER]);
+		kw[BLAYER] = gtk_range_get_value(scale_white_balance[BLAYER]);
 	} else {
-		get_coeff_for_wb(fit, white_selection, coef);
+		low = gtk_range_get_value(scaleLimit[0]);
+		high = gtk_range_get_value(scaleLimit[1]);
+		get_coeff_for_wb(fit, white_selection, black_selection, kw, bg, &norm, low, high);
 	}
-#pragma omp parallel for num_threads(com.max_thread) private(chan) schedule(dynamic, 1)
-	for (chan = 0; chan < nb_chan; chan++) {
-		fmul(fit, chan, coef[chan]);
+#pragma omp parallel for num_threads(com.max_thread) private(chan) schedule(static)
+	for (chan = 0; chan < 3; chan++) {
+		if (kw[chan] == 1.0) continue;
+		calibrate(fit, chan, kw[chan], bg[chan], norm);
 	}
 }
 
@@ -790,7 +817,6 @@ void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data)
 
 	siril_log_color_message("Color Calibration: processing...\n", "red");
 	gettimeofday(&t_start, NULL);
-	undo_save_state("Processing: Color Calibration");
 
 	GtkToggleButton *manual = GTK_TOGGLE_BUTTON(
 			lookup_widget("checkbutton_manual_calibration"));
@@ -834,22 +860,22 @@ void on_calibration_apply_button_clicked(GtkButton *button, gpointer user_data)
 	white_selection.w = gtk_spin_button_get_value(selection_white_value[2]);
 	white_selection.h = gtk_spin_button_get_value(selection_white_value[3]);
 
-	if (!white_selection.w || !white_selection.h) {
-		white_selection.x = 0;
-		white_selection.y = 0;
-		white_selection.w = gfit.rx;
-		white_selection.h = gfit.ry;
+	if ((!white_selection.w || !white_selection.h) && !is_manual) {
+		show_dialog("Make a selection of the white area before", "Warning",
+				"gtk-dialog-warning");
+		return;
 	}
 
 	set_cursor_waiting(TRUE);
+	undo_save_state("Processing: Color Calibration");
 	white_balance(&gfit, is_manual, white_selection, black_selection);
-	background_neutralize(&gfit, black_selection);
-	delete_selected_area();
 
 	gettimeofday(&t_end, NULL);
 
 	show_time(t_start, t_end);
 
+	delete_selected_area();
+
 	redraw(com.cvport, REMAP_ALL);
 	redraw_previews();
 	set_cursor_waiting(FALSE);
