@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2016 team free-astro (see more in AUTHORS file)
  * Reference site is http://free-astro.vinvin.tf/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -29,7 +29,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <assert.h>
-#include <signal.h>
 #if (defined(__APPLE__) && defined(__MACH__))
 #include <stdlib.h>
 #include <libproc.h>
@@ -44,6 +43,8 @@
 #include "core/undo.h"
 #include "io/single_image.h"
 
+#define GLADE_FILE "siril3.glade"
+
 /* the global variables of the whole project */
 cominfo com;	// the main data struct
 fits gfit;	// currently loaded image
@@ -73,7 +74,7 @@ void usage(const char *command) {
 void signal_handled(int s) {
 	//printf("Caught signal %d\n", s);
 	undo_flush();
-	exit(1);
+	exit(EXIT_FAILURE);
 
 }
 
@@ -116,7 +117,7 @@ int main(int argc, char *argv[]) {
 						c='f';					
 					else {
 						usage(argv[0]);
-						exit(1);
+						exit(EXIT_FAILURE);
 					}
 				}
 			}
@@ -129,18 +130,18 @@ int main(int argc, char *argv[]) {
 				break;
 			case 'v':
 				fprintf(stdout, "%s %s\n", PACKAGE, VERSION);
-				exit(1);
+				exit(EXIT_SUCCESS);
 				break;
 			case 'f':
 				list_format_available();
-				exit(1);
+				exit(EXIT_SUCCESS);
 				break;
 			default:
 				fprintf(stderr, "unknown command line parameter `%c'\n", c);
 				/* no break */
 			case 'h':
 				usage(argv[0]);
-				exit(1);
+				exit(EXIT_SUCCESS);
 		}
 	}
 
@@ -169,21 +170,22 @@ int main(int argc, char *argv[]) {
 	do {
 		GError *err = NULL;
 		GString *pathStr = g_string_new (siril_sources[i]);
-		g_string_append(pathStr, "siril3.glade");
+		g_string_append(pathStr, GLADE_FILE);
 		gchar *path = g_string_free (pathStr, FALSE);
 
 		if (gtk_builder_add_from_file (builder, path, &err)) {
-			fprintf(stdout, "successfully loaded %s\n", siril_sources[i]);
+			fprintf(stdout, "Successfully loaded '%s%s'\n", siril_sources[i], GLADE_FILE);
 			g_free(path);
 			break;
 		}
+		fprintf (stderr, "Unable to read file: %s\n", err->message);
 		g_free(path);
 		g_error_free(err);
 		i++;
 	} while (i < sizeof(siril_sources)/sizeof(char *));
 	if (i == sizeof(siril_sources) / sizeof(char *)) {
-		fprintf(stderr, "siril3.glade was not found or contains errors, cannot render GUI. Exiting.\n");
-		exit(1);
+		fprintf(stderr, "%s was not found or contains errors, cannot render GUI. Exiting.\n", GLADE_FILE);
+		exit(EXIT_FAILURE);
 	}
 	siril_path = siril_sources[i];
 
@@ -200,6 +202,7 @@ int main(int argc, char *argv[]) {
 	gtk_text_buffer_create_tag (tbuf, "salmon", "foreground", "#ff9898", NULL);
 	gtk_text_buffer_create_tag (tbuf, "green", "foreground", "#01b301", NULL);
 	gtk_text_buffer_create_tag (tbuf, "blue", "foreground", "#7a7af8", NULL);
+	gtk_text_buffer_create_tag (tbuf, "plum", "foreground", "#8e4585", NULL);
 	
 	siril_log_color_message("Welcome to %s v%s\n", "bold", PACKAGE, VERSION);
 
@@ -279,9 +282,6 @@ int main(int argc, char *argv[]) {
 
 	g_free(csspath);
 
-	/* initialize prepreocess-related stuff */
-	initialize_preprocessing();
-	
 	/* initialize registration methods */
 	initialize_registration_methods();
 
@@ -356,4 +356,3 @@ void initialize_scrollbars() {
 				G_CALLBACK(scrollbars_vadjustment_changed_handler), NULL);
 	}
 }
-
