@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2016 team free-astro (see more in AUTHORS file)
  * Reference site is http://free-astro.vinvin.tf/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 #include "registration/registration.h"
 #include "registration/matching/misc.h"
 #include "registration/matching/match.h"
+#include "registration/matching/atpmatch.h"
 #include "algos/star_finder.h"
 #include "stacking/stacking.h"
 #include "core/siril.h"
@@ -39,41 +40,53 @@
 #include "algos/PSF.h"
 #include "gui/PSF_list.h"
 #include "algos/quality.h"
+#include "io/ser.h"
 #ifdef HAVE_OPENCV
 #include "opencv/opencv.h"
+#include "opencv/ecc/ecc.h"
 #endif
 
-#define MAX_STARS_FITTED 100
+#define MAX_STARS_FITTED 2000
 #undef DEBUG
 
-static char *tooltip_text[] = { "Image pattern alignment: This is a simple registration by translation method "
-		"using cross correlation in the spatial domain. This method is fast and is used to register "
-		"planetary movies. It can also be used for some deep-sky images registration. "
-		"Shifts at pixel precision are saved in seq file.",
-		"One star registration: This is the simplest method to register deep-sky images. "
+static char *tooltip_text[] = { "One Star Registration: This is the simplest method to register deep-sky images. "
 		"Because only one star is concerned for register, images are aligned using shifting"
 		"(at a fraction of pixel). No rotation or scaling are performed. "
 		"Shifts at pixel precision are saved in seq file.",
-		"Global star alignment: This is a more powerfull and accurate algorithm (but also slower) "
+#ifdef HAVE_OPENCV
+		"Global Star Alignment: This is a more powerful and accurate algorithm (but also slower) "
 		"to perform deep-sky images. The global matching is based on triangle similarity method for automatically "
-		"identify commom stars in each image."
-		"A new sequence is created with the prefix of your choice (r_ by default). "
+		"identify common stars in each image."
+		"A new sequence is created with the prefix of your choice (r_ by default). ",
+#endif
+		"Image Pattern Alignment: This is a simple registration by translation method "
+		"using cross correlation in the spatial domain. This method is fast and is used to register "
+		"planetary movies. It can also be used for some deep-sky images registration. "
+		"Shifts at pixel precision are saved in seq file."
+#ifdef HAVE_OPENCV
+,
+		"Enhanced Correlation Coefficient Maximization: This method is based on the enhanced correlation "
+		"coefficient maximization algorithm. This method is more complex and slower than Image Pattern Alignment "
+		"but no selection is required. It is good for moon surface images registration. Only translation is taken "
+		"into account yet."
+#endif
 };
 /* callback for the selected area event */
 void _reg_selected_area_callback() {
 	update_reg_interface(TRUE);
 }
 
-static struct registration_method *reg_methods[4];
+static struct registration_method *reg_methods[NUMBER_OF_METHOD];
 static gpointer register_thread_func(gpointer p);
 static gboolean end_register_idle(gpointer p);
 
 struct registration_method *new_reg_method(char *name, registration_function f,
-		selection_type s) {
+		selection_type s, registration_type t) {
 	struct registration_method *reg = malloc(sizeof(struct registration_method));
 	reg->name = strdup(name);
 	reg->method_ptr = f;
 	reg->sel = s;
+	reg->type = t;
 	return reg;
 }
 
@@ -83,13 +96,15 @@ void initialize_registration_methods() {
 	GString *tip;
 	gchar *ctip;
 
-	reg_methods[i++] = new_reg_method("Image pattern alignment (planetary/deep-sky)",
-			&register_shift_dft, REQUIRES_SQUARED_SELECTION);
-	reg_methods[i++] = new_reg_method("One star registration (deep-sky)",
-			&register_shift_fwhm, REQUIRES_ANY_SELECTION);
+	reg_methods[i++] = new_reg_method("One Star Registration (deep-sky)",
+			&register_shift_fwhm, REQUIRES_ANY_SELECTION, REGTYPE_DEEPSKY);
 #ifdef HAVE_OPENCV
-	reg_methods[i++] = new_reg_method("Global star alignement (deep-sky)",
-			&register_star_alignment, REQUIRES_NO_SELECTION);
+	reg_methods[i++] = new_reg_method("Global Star Alignment (deep-sky)",
+			&register_star_alignment, REQUIRES_NO_SELECTION, REGTYPE_DEEPSKY);
+	reg_methods[i++] = new_reg_method("Image Pattern Alignment (planetary - Full Disk)",
+			&register_shift_dft, REQUIRES_SQUARED_SELECTION, REGTYPE_PLANETARY);
+	reg_methods[i++] = new_reg_method("Enhanced Correlation Coefficient (planetary - Surfaces)",
+			&register_ecc, REQUIRES_NO_SELECTION, REGTYPE_PLANETARY);
 #endif
 	//if (theli_is_available())
 	//	reg_methods[i++] = new_reg_method("theli", register_theli, REQUIRES_NO_SELECTION);
@@ -125,34 +140,53 @@ void initialize_registration_methods() {
 }
 
 struct registration_method *get_selected_registration_method() {
-	GtkComboBox *regcombo = GTK_COMBO_BOX(
+	GtkComboBoxText *regcombo = GTK_COMBO_BOX_TEXT(
 			gtk_builder_get_object(builder, "comboboxregmethod"));
-	int index = gtk_combo_box_get_active(regcombo);
+	int index = 0;
+
+	gchar *text = gtk_combo_box_text_get_active_text (regcombo);
+	while (reg_methods[index] && text != NULL) {
+		if (!strcmp(reg_methods[index]->name, text))
+			break;
+		index++;
+	}
+	g_free(text);
 	return reg_methods[index];
 }
 
+static void normalizeQualityData(struct registration_args *args, double q_min, double q_max) {
+	int frame;
+
+	for (frame = 0; frame < args->seq->number; ++frame) {
+		if (args->run_in_thread && !get_thread_run()) {
+			break;
+		}
+		if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
+			continue;
+
+		args->seq->regparam[args->layer][frame].quality -= q_min;
+		args->seq->regparam[args->layer][frame].quality /= (q_max - q_min);
+	}
+}
+
 /* register images: calculate shift in images to be aligned with the reference image;
  * images are not modified, only shift parameters are saved in regparam in the sequence.
  * layer is the layer on which the registration will be done, green by default (set in siril_init())
  */
 int register_shift_dft(struct registration_args *args) {
-	int i, x, size, sqsize;
-	fftw_complex *ref, *img, *in, *out, *convol;
+	fits fit_ref;
+	int frame, size, sqsize;
+	fftw_complex *ref, *in, *out, *convol;
 	fftw_plan p, q;
-	int shiftx, shifty, shift;
-	int ret;
+	int ret, j;
 	int plan;
-	fftw_complex *cbuf, *ibuf, *obuf;
+	int abort = 0;
 	float nb_frames, cur_nb;
 	int ref_image;
 	regdata *current_regdata;
-	char tmpmsg[1024], tmpfilename[256];
 	rectangle full_area;	// the area to use after getting image_part
-	int j;
-	double q_max = 0;
+	double q_max = 0, q_min = DBL_MAX;
 	int q_index = -1;
-	fits fit;
-	memset(&fit, 0, sizeof(fits));
 
 	/* the selection needs to be squared for the DFT */
 	assert(args->selection.w == args->selection.h);
@@ -191,21 +225,19 @@ int register_shift_dft(struct registration_args *args) {
 	set_progress_bar_data(
 			"Register DFT: loading and processing reference frame",
 			PROGRESS_NONE);
-	ret = seq_read_frame_part(args->seq, args->layer, ref_image, &fit,
+	memset(&fit_ref, 0, sizeof(fits));
+	ret = seq_read_frame_part(args->seq, args->layer, ref_image, &fit_ref,
 			&args->selection);
 
-	q_max = current_regdata[ref_image].quality;
-	q_index = ref_image;
-
 	if (ret) {
 		siril_log_message(
 				"Register: could not load first image to register, aborting.\n");
 		free(current_regdata);
+		clearfits(&fit_ref);
 		return ret;
 	}
 
 	ref = fftw_malloc(sizeof(fftw_complex) * sqsize);
-	img = fftw_malloc(sizeof(fftw_complex) * sqsize);
 	in = fftw_malloc(sizeof(fftw_complex) * sqsize);
 	out = fftw_malloc(sizeof(fftw_complex) * sqsize);
 	convol = fftw_malloc(sizeof(fftw_complex) * sqsize);
@@ -220,90 +252,122 @@ int register_shift_dft(struct registration_args *args) {
 
 	// copying image selection into the fftw data
 	for (j = 0; j < sqsize; j++)
-		ref[j] = (double) fit.data[j];
+		ref[j] = (double) fit_ref.data[j];
 
 	// We don't need fit anymore, we can destroy it.
-	current_regdata[ref_image].quality = QualityEstimate(&fit, args->layer, QUALTYPE_NORMAL);
-	clearfits(&fit);
+	current_regdata[ref_image].quality = QualityEstimate(&fit_ref, args->layer, QUALTYPE_NORMAL);
+	clearfits(&fit_ref);
 	fftw_execute_dft(p, ref, in); /* repeat as needed */
 	current_regdata[ref_image].shiftx = 0;
 	current_regdata[ref_image].shifty = 0;
 
-	for (i = 0, cur_nb = 0.f; i < args->seq->number; ++i) {
-		if (args->run_in_thread && !get_thread_run())
-			break;
-		if (i == ref_image)
-			continue;
-		if (!args->process_all_frames && !args->seq->imgparam[i].incl)
-			continue;
+	q_min = q_max = current_regdata[ref_image].quality;
+	q_index = ref_image;
 
-		seq_get_image_filename(args->seq, i, tmpfilename);
-		g_snprintf(tmpmsg, 1024, "Register: processing image %s\n", tmpfilename);
-		set_progress_bar_data(tmpmsg, PROGRESS_NONE);
-		if (!(ret = seq_read_frame_part(args->seq, args->layer, i, &fit,
-				&args->selection))) {
-
-			// copying image selection into the fftw data
-			for (j = 0; j < sqsize; j++)
-				img[j] = (double) fit.data[j];
-
-			// We don't need fit anymore, we can destroy it.
-			current_regdata[i].quality = QualityEstimate(&fit, args->layer, QUALTYPE_NORMAL);
-
-			if (current_regdata[i].quality > q_max) {
-				q_max = current_regdata[i].quality;
-				q_index = i;
+	cur_nb = 0.f;
+#pragma omp parallel for num_threads(com.max_thread) private(frame) schedule(static) \
+	if((args->seq->type == SEQ_REGULAR && fits_is_reentrant()) || args->seq->type == SEQ_SER)
+	for (frame = 0; frame < args->seq->number; ++frame) {
+		if (!abort) {
+			if (args->run_in_thread && !get_thread_run()) {
+				abort = 1;
+				continue;
 			}
+			if (frame == ref_image)
+				continue;
+			if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
+				continue;
 
-			clearfits(&fit);
-			fftw_execute_dft(p, img, out); /* repeat as needed */
-			/* originally, quality is computed with the quality
-			 * function working on the fft space. Now we use quality
-			 * instead.
-			 */
-			cbuf = convol;
-			ibuf = in;
-			obuf = out;
-			for (x = 0; x < sqsize; x++) {
-				*cbuf++ = *ibuf++ * conj(*obuf++);
-			}
+			char tmpmsg[1024], tmpfilename[256];
+			fits fit;
+			memset(&fit, 0, sizeof(fits));
 
-			fftw_execute_dft(q, convol, ref); /* repeat as needed */
-			shift = 0;
-			for (x = 1; x < sqsize; ++x) {
-				if (creal(ref[x]) > creal(ref[shift])) {
-					shift = x;
-					// break or get last value?
+			seq_get_image_filename(args->seq, frame, tmpfilename);
+			g_snprintf(tmpmsg, 1024, "Register: processing image %s\n",
+					tmpfilename);
+			set_progress_bar_data(tmpmsg, PROGRESS_NONE);
+			if (!(seq_read_frame_part(args->seq, args->layer, frame, &fit,
+					&args->selection))) {
+
+				int x;
+				fftw_complex *img = fftw_malloc(sizeof(fftw_complex) * sqsize);
+				fftw_complex *out2 = fftw_malloc(sizeof(fftw_complex) * sqsize);
+
+				// copying image selection into the fftw data
+				for (x = 0; x < sqsize; x++)
+					img[x] = (double) fit.data[x];
+
+				// We don't need fit anymore, we can destroy it.
+				current_regdata[frame].quality = QualityEstimate(&fit, args->layer,
+						QUALTYPE_NORMAL);
+
+				clearfits(&fit);
+
+#pragma omp critical
+				{
+					double qual = current_regdata[frame].quality;
+					if (qual > q_max) {
+						q_max = qual;
+						q_index = frame;
+					}
+					q_min = min(q_min, qual);
 				}
-			}
-			shifty = shift / size;
-			shiftx = shift % size;
-			if (shifty > size / 2) {
-				shifty -= size;
-			}
-			if (shiftx > size / 2) {
-				shiftx -= size;
-			}
 
-			current_regdata[i].shiftx = shiftx;
-			current_regdata[i].shifty = shifty;
+				fftw_execute_dft(p, img, out2); /* repeat as needed */
 
-			/* shiftx and shifty are the x and y values for translation that
-			 * would make this image aligned with the reference image.
-			 * WARNING: the y value is counted backwards, since the FITS is
-			 * stored down from up.
-			 */
-			fprintf(stderr, "reg: file %d, shiftx=%d shifty=%d quality=%g\n",
-					args->seq->imgparam[i].filenum, current_regdata[i].shiftx,
-					current_regdata[i].shifty, current_regdata[i].quality);
-			cur_nb += 1.f;
-			set_progress_bar_data(NULL, cur_nb / nb_frames);
-		} else {
-			//report_fits_error(ret, error_buffer);
-			if (current_regdata == args->seq->regparam[args->layer])
-				args->seq->regparam[args->layer] = NULL;
-			free(current_regdata);
-			return ret;
+				fftw_complex *convol2 = fftw_malloc(sizeof(fftw_complex) * sqsize);
+
+				for (x = 0; x < sqsize; x++) {
+					convol2[x] = in[x] * conj(out2[x]);
+				}
+
+				fftw_execute_dft(q, convol2, out2); /* repeat as needed */
+				fftw_free(convol2);
+
+				int shift = 0;
+				for (x = 1; x < sqsize; ++x) {
+					if (creal(out2[x]) > creal(out2[shift])) {
+						shift = x;
+						// break or get last value?
+					}
+				}
+				int shifty = shift / size;
+				int shiftx = shift % size;
+				if (shifty > size / 2) {
+					shifty -= size;
+				}
+				if (shiftx > size / 2) {
+					shiftx -= size;
+				}
+
+				current_regdata[frame].shiftx = shiftx;
+				current_regdata[frame].shifty = shifty;
+
+				/* shiftx and shifty are the x and y values for translation that
+				 * would make this image aligned with the reference image.
+				 * WARNING: the y value is counted backwards, since the FITS is
+				 * stored down from up.
+				 */
+#ifdef DEBUG
+				fprintf(stderr,
+						"reg: frame %d, shiftx=%d shifty=%d quality=%g\n",
+						args->seq->imgparam[frame].filenum,
+						current_regdata[frame].shiftx, current_regdata[frame].shifty,
+						current_regdata[frame].quality);
+#endif
+#pragma omp atomic
+				cur_nb += 1.f;
+				set_progress_bar_data(NULL, cur_nb / nb_frames);
+				fftw_free(img);
+				fftw_free(out2);
+			} else {
+				//report_fits_error(ret, error_buffer);
+				if (current_regdata == args->seq->regparam[args->layer])
+					args->seq->regparam[args->layer] = NULL;
+				free(current_regdata);
+				abort = ret = 1;
+				continue;
+			}
 		}
 	}
 
@@ -312,14 +376,15 @@ int register_shift_dft(struct registration_args *args) {
 	fftw_free(in);
 	fftw_free(out);
 	fftw_free(ref);
-	fftw_free(img);
 	fftw_free(convol);
-	args->seq->regparam[args->layer] = current_regdata;
-	update_used_memory();
-	siril_log_message("Registration finished.\n");
-	siril_log_color_message("Best frame: #%d with quality=%g.\n", "bold",
-			q_index, q_max);
-	return 0;
+	if (!ret) {
+		args->seq->regparam[args->layer] = current_regdata;
+		normalizeQualityData(args, q_min, q_max);
+		update_used_memory();
+		siril_log_message("Registration finished.\n");
+		siril_log_color_message("Best frame: #%d.\n", "bold", q_index);
+	}
+	return ret;
 }
 
 /* register images: calculate shift in images to be aligned with the reference image;
@@ -337,7 +402,8 @@ int register_shift_fwhm(struct registration_args *args) {
 	 * images to register, which provides FWHM but also star coordinates */
 	// TODO: detect that it was already computed, and don't do it again
 	// -> should be done at a higher level and passed in the args
-	if (do_fwhm_sequence_processing(args->seq, args->layer))	// stores in regparam
+	if (do_fwhm_sequence_processing(args->seq, args->layer, 0,
+			args->run_in_thread))	// stores in regparam
 		return 1;
 
 	current_regdata = args->seq->regparam[args->layer];
@@ -409,12 +475,13 @@ int register_shift_fwhm(struct registration_args *args) {
 }
 
 #ifdef HAVE_OPENCV
-void _print_result(TRANS *trans, float FWHMx, float FWHMy) {
+
+static void _print_result(TRANS *trans, float FWHMx, float FWHMy) {
 	double rotation, scale;
 	point shift;
 
 	switch (trans->order) {
-	case 1:
+	case AT_TRANS_LINEAR:
 		rotation = atan2(trans->c, trans->b);
 		shift.x = trans->a;
 		shift.y = -trans->d;
@@ -435,7 +502,7 @@ void _print_result(TRANS *trans, float FWHMx, float FWHMy) {
 
 int register_star_alignment(struct registration_args *args) {
 	int frame, ref_image, ret, i;
-	int fitted_stars;
+	int fitted_stars, failed = 0;
 	float nb_frames, cur_nb;
 	float FWHMx, FWHMy;
 	fitted_PSF **stars;
@@ -443,8 +510,9 @@ int register_star_alignment(struct registration_args *args) {
 	regdata *current_regdata;
 	starFinder sf;
 	fits fit;
-	memset(&fit, 0, sizeof(fits));
+	struct ser_struct *new_ser;
 
+	memset(&fit, 0, sizeof(fits));
 	memset(&sf, 0, sizeof(starFinder));
 
 	if (!args->seq->regparam) {
@@ -485,7 +553,7 @@ int register_star_alignment(struct registration_args *args) {
 	}
 	siril_log_color_message("Reference Image:\n", "green");
 	com.stars = peaker(&fit, args->layer, &sf);
-	if (sf.nb_stars < 4) {
+	if (sf.nb_stars < AT_MATCH_MINPAIRS) {
 		siril_log_message(
 				"There are not enough stars in reference image to perform alignment\n");
 		if (current_regdata == args->seq->regparam[args->layer])
@@ -495,20 +563,42 @@ int register_star_alignment(struct registration_args *args) {
 	}
 	redraw(com.cvport, REMAP_NONE); // draw stars
 #ifdef DEBUG
-	printf("REFERENCE IMAGE\n");
-	for (i = 0; i < MAX_STARS_FITTED; i++) {
-		printf("%.3lf\t%.3lf\t%.3lf\n",
-				com.stars[i]->xpos, com.stars[i]->ypos, com.stars[i]->mag);
-	}
+		FILE *pfile;
+
+		pfile = fopen("ref.txt", "w+");
+		fprintf(pfile, "REFERENCE IMAGE\n");
+		for (i = 0; i < MAX_STARS_FITTED; i++) {
+			fprintf(pfile, "%.3lf\t%.3lf\t%.3lf\n",
+					com.stars[i]->xpos, com.stars[i]->ypos, com.stars[i]->mag);
+		}
+		fclose(pfile);
 #endif
-	fitted_stars = (sf.nb_stars > MAX_STARS_FITTED) ? MAX_STARS_FITTED : sf.nb_stars;
+	fitted_stars =
+			(sf.nb_stars > MAX_STARS_FITTED) ? MAX_STARS_FITTED : sf.nb_stars;
 	FWHM_average(com.stars, &FWHMx, &FWHMy, fitted_stars);
 	siril_log_message("FWHMx:\t%*.2f px\n", 8, FWHMx);
 	siril_log_message("FWHMy:\t%*.2f px\n", 8, FWHMy);
 	current_regdata[ref_image].fwhm = FWHMx;
 
 	/* then we compare to other frames */
-	args->seq->new_total = args->seq->number;
+	if (args->process_all_frames)
+		args->seq->new_total = args->seq->number;
+	else args->seq->new_total = args->seq->selnum;
+
+	if (args->seq->type == SEQ_SER) {
+		char dest[256];
+
+		new_ser = malloc(sizeof(struct ser_struct));
+
+		const char *ptr = strrchr(args->seq->seqname, '/');
+		if (ptr)
+			snprintf(dest, 255, "%s%s.ser", args->text, ptr + 1);
+		else
+			snprintf(dest, 255, "%s%s.ser", args->text, args->seq->seqname);
+
+		ser_create_file(dest, new_ser, TRUE, args->seq->ser_file);
+	}
+
 	for (frame = 0, cur_nb = 0.f; frame < args->seq->number; frame++) {
 		if (args->run_in_thread && !get_thread_run())
 			break;
@@ -522,20 +612,26 @@ int register_star_alignment(struct registration_args *args) {
 
 			if (frame != ref_image) {
 				stars = peaker(&fit, args->layer, &sf);
-				if (sf.nb_stars < 4) {
+				if (sf.nb_stars < AT_MATCH_MINPAIRS) {
 					siril_log_message("Not enough stars. Image %d skipped\n",
 							frame);
 					args->seq->new_total--;
+					failed++;
 					continue;
 				}
 
 #ifdef DEBUG
-				printf("IMAGE %d\n", frame);
+				FILE *pfile2;
+
+				pfile2 = fopen("im.txt", "w+");
+				fprintf(pfile2, "IMAGE %d\n", frame);
 				for (i = 0; i < MAX_STARS_FITTED; i++) {
-					printf("%.3lf\t%.3lf\t%.3lf\n", stars[i]->xpos,
+					fprintf(pfile2, "%.3lf\t%.3lf\t%.3lf\n", stars[i]->xpos,
 							stars[i]->ypos, stars[i]->mag);
 				}
-				printf("\n\n", frame);
+				fprintf(pfile2, "\n\n", frame);
+				fclose(pfile2);
+
 #endif
 
 				nbpoints = (sf.nb_stars < fitted_stars) ?
@@ -546,6 +642,7 @@ int register_star_alignment(struct registration_args *args) {
 							"Cannot perform star matching. Image %d skipped\n",
 							"red", frame);
 					args->seq->new_total--;
+					failed++;
 					i = 0;
 					while (i < MAX_STARS && stars[i])
 						free(stars[i++]);
@@ -557,7 +654,7 @@ int register_star_alignment(struct registration_args *args) {
 				_print_result(&trans, FWHMx, FWHMy);
 				current_regdata[frame].fwhm = FWHMx;
 
-				transforme_image(&fit, trans, 0);
+				cvTransformImage(&fit, trans, 0);
 
 				i = 0;
 				while (i < MAX_STARS && stars[i])
@@ -566,57 +663,194 @@ int register_star_alignment(struct registration_args *args) {
 			}
 			fit_sequence_get_image_filename(args->seq, frame, filename, TRUE);
 
-			snprintf(dest, 255, "%s%s", args->text, filename);
-			savefits(dest, &fit);
+			if (args->seq->type == SEQ_SER)
+				ser_write_frame_from_fit(new_ser, &fit, frame);
+			else {
+				snprintf(dest, 256, "%s%s", args->text, filename);
+				savefits(dest, &fit);
+			}
 
 			cur_nb += 1.f;
 			set_progress_bar_data(NULL, cur_nb / nb_frames);
 		}
 	}
+	if (args->seq->type == SEQ_SER) {
+		ser_write_and_close(new_ser);
+		free(new_ser);
+	}
 	args->seq->regparam[args->layer] = current_regdata;
 	update_used_memory();
 	siril_log_message("Registration finished.\n");
+	siril_log_color_message("%d images processed.\n", "green", args->seq->new_total + failed);
+	siril_log_color_message("Total: %d failed, %d registred.\n", "green", failed, args->seq->new_total);
+
+	return args->seq->new_total == 0;
+}
+
+int register_ecc(struct registration_args *args) {
+	int frame, ref_image, ret, failed = 0;
+	float nb_frames, cur_nb;
+	regdata *current_regdata;
+	fits ref;
+	double q_max = 0, q_min = DBL_MAX;
+	int q_index = -1;
+	int abort = 0;
+
+	if (!args->seq->regparam) {
+		fprintf(stderr, "regparam should have been created before\n");
+		return -1;
+	}
+	if (args->seq->regparam[args->layer]) {
+		current_regdata = args->seq->regparam[args->layer];
+		/* we reset all values as we built another sequence */
+		memset(current_regdata, 0, args->seq->number * sizeof(regdata));
+	} else {
+		current_regdata = calloc(args->seq->number, sizeof(regdata));
+		if (current_regdata == NULL) {
+			siril_log_message("Error allocating registration data\n");
+			return -2;
+		}
+	}
+
+	if (args->process_all_frames)
+		nb_frames = (float) args->seq->number;
+	else
+		nb_frames = (float) args->seq->selnum;
+
+	/* loading reference frame */
+	if (args->seq->reference_image == -1)
+		ref_image = 0;
+	else
+		ref_image = args->seq->reference_image;
+
+	memset(&ref, 0, sizeof(fits));
+
+	/* first we're looking for stars in reference image */
+	ret = seq_read_frame(args->seq, ref_image, &ref);
+	if (ret) {
+		siril_log_message("Could not load reference image\n");
+		if (current_regdata == args->seq->regparam[args->layer])
+			args->seq->regparam[args->layer] = NULL;
+		free(current_regdata);
+		return 1;
+	}
+	current_regdata[ref_image].quality = QualityEstimate(&ref, args->layer, QUALTYPE_NORMAL);
+	/* we make sure to free data in the destroyed fit */
+	clearfits(&ref);
+	/* Ugly code: as QualityEstimate destroys fit we need to reload it */
+	seq_read_frame(args->seq, ref_image, &ref);
+	q_min = q_max = current_regdata[ref_image].quality;
+	q_index = ref_image;
+
+	/* then we compare to other frames */
+	if (args->process_all_frames)
+		args->seq->new_total = args->seq->number;
+	else args->seq->new_total = args->seq->selnum;
+
+	cur_nb = 0.f;
+#pragma omp parallel for num_threads(com.max_thread) private(frame) schedule(static) \
+	if((args->seq->type == SEQ_REGULAR && fits_is_reentrant()) || args->seq->type == SEQ_SER)
+	for (frame = 0; frame < args->seq->number; frame++) {
+		if (!abort) {
+			if (args->run_in_thread && !get_thread_run()) {
+				abort = 1;
+				continue;
+			}
+			if (!args->process_all_frames && !args->seq->imgparam[frame].incl)
+				continue;
+			current_regdata[frame].shiftx = 0;
+			current_regdata[frame].shifty = 0;
+
+			char tmpmsg[1024], tmpfilename[256];
+
+			seq_get_image_filename(args->seq, frame, tmpfilename);
+			g_snprintf(tmpmsg, 1024, "Register: processing image %s\n",
+					tmpfilename);
+			set_progress_bar_data(tmpmsg, PROGRESS_NONE);
+
+			if (frame != ref_image) {
+				fits im;
+
+				memset(&im, 0, sizeof(fits));
+				ret = seq_read_frame(args->seq, frame, &im);
+				if (!ret) {
+					reg_ecc reg_param;
+					memset(&reg_param, 0, sizeof(reg_ecc));
+
+					if (findTransform(&ref, &im, args->layer, &reg_param)) {
+						siril_log_message(
+								"Cannot perform ECC alignment for frame %d\n",
+								frame);
+						/* We exclude this frame */
+						com.seq.imgparam[frame].incl = FALSE;
+#pragma omp atomic
+						++failed;
+						clearfits(&im);
+						continue;
+					}
+
+					current_regdata[frame].quality = QualityEstimate(&im,
+							args->layer, QUALTYPE_NORMAL);
+
+#pragma omp critical
+					{
+						double qual = current_regdata[frame].quality;
+						if (qual > q_max) {
+							q_max = qual;
+							q_index = frame;
+						}
+						q_min = min(q_min, qual);
+					}
+
+					current_regdata[frame].shiftx = -round_to_int(reg_param.dx);
+					current_regdata[frame].shifty = -round_to_int(reg_param.dy);
+
+#pragma omp atomic
+					cur_nb += 1.f;
+					set_progress_bar_data(NULL, cur_nb / nb_frames);
+					clearfits(&im);
+				}
+			}
+		}
+	}
+	args->seq->regparam[args->layer] = current_regdata;
+	normalizeQualityData(args, q_min, q_max);
+	clearfits(&ref);
+	update_used_memory();
+	siril_log_message("Registration finished.\n");
+	if (failed)
+		siril_log_color_message("%d frames were excluded.\n", "red", failed);
+	siril_log_color_message("Best frame: #%d.\n", "bold", q_index);
 
 	return 0;
 }
+
 #endif
 
 void on_comboboxregmethod_changed(GtkComboBox *box, gpointer user_data) {
-	int reg = gtk_combo_box_get_active(box);
+	int index = 0;
+	gchar *text = gtk_combo_box_text_get_active_text (GTK_COMBO_BOX_TEXT(box));
 
-	com.reg_settings = reg;
+	while (reg_methods[index] && text != NULL) {
+		if (!strcmp(reg_methods[index]->name, text))
+			break;
+		index++;
+	}
+	g_free(text);
+
+	com.reg_settings = index;
 	update_reg_interface(TRUE);
 	writeinitfile();
 }
 
-int get_registration_layer(sequence *seq) {
+int get_registration_layer() {
 	int reglayer;
-	gboolean has_changed = FALSE;
 	GtkComboBox *registbox = GTK_COMBO_BOX(lookup_widget("comboboxreglayer"));
 
 	if (!sequence_is_loaded())
 		return -1;
 	reglayer = gtk_combo_box_get_active(registbox);
 
-	if (seq->nb_layers == 3) {
-		if (!seq->regparam[reglayer]) {
-			if (seq->regparam[GLAYER]) {
-				reglayer = GLAYER;
-				has_changed = TRUE;
-			}
-			else if (seq->regparam[RLAYER]) {
-				reglayer = RLAYER;
-				has_changed = TRUE;
-			}
-			else if (seq->regparam[BLAYER]) {
-				reglayer = BLAYER;
-				has_changed = TRUE;
-			}
-		}
-		if (has_changed) {
-			gtk_combo_box_set_active(registbox, reglayer);
-		}
-	}
 	return reglayer;
 }
 
@@ -625,7 +859,7 @@ int get_registration_layer(sequence *seq) {
  * Verifies that enough images are selected and an area is selected.
  */
 void update_reg_interface(gboolean dont_change_reg_radio) {
-	static GtkWidget *go_register = NULL, *w_entry = NULL;
+	static GtkWidget *go_register = NULL, *newSequence = NULL;
 	static GtkLabel *labelreginfo = NULL;
 	static GtkToggleButton *reg_all = NULL, *reg_sel = NULL;
 	int nb_images_reg; /* the number of images to register */
@@ -639,7 +873,7 @@ void update_reg_interface(gboolean dont_change_reg_radio) {
 				gtk_builder_get_object(builder, "regselbutton"));
 		labelreginfo = GTK_LABEL(
 				gtk_builder_get_object(builder, "labelregisterinfo"));
-		w_entry = lookup_widget("regseqname_entry");
+		newSequence = lookup_widget("box29");
 	}
 
 	if (!dont_change_reg_radio) {
@@ -661,11 +895,11 @@ void update_reg_interface(gboolean dont_change_reg_radio) {
 		gtk_widget_set_sensitive(go_register, TRUE);
 		gtk_label_set_text(labelreginfo, "");
 #ifdef HAVE_OPENCV
-		gtk_widget_set_sensitive(w_entry, method->method_ptr == &register_star_alignment);
+		gtk_widget_set_visible(newSequence, method->method_ptr == &register_star_alignment);
 #endif
 	} else {
 		gtk_widget_set_sensitive(go_register, FALSE);
-		gtk_widget_set_sensitive(w_entry, FALSE);
+		gtk_widget_set_visible(newSequence, FALSE);
 		if (nb_images_reg <= 1 && com.selection.w <= 0 && com.selection.h <= 0)
 			if (!sequence_is_loaded())
 				gtk_label_set_text(labelreginfo, "Load a sequence first.");
