@@ -1,7 +1,7 @@
 /*
  * This file is part of Siril, an astronomy image processor.
  * Copyright (C) 2005-2011 Francois Meyer (dulle at free.fr)
- * Copyright (C) 2012-2015 team free-astro (see more in AUTHORS file)
+ * Copyright (C) 2012-2016 team free-astro (see more in AUTHORS file)
  * Reference site is http://free-astro.vinvin.tf/index.php/Siril
  *
  * Siril is free software: you can redistribute it and/or modify
@@ -28,9 +28,6 @@
 #ifdef _OPENMP
 #include <omp.h>
 #endif
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
 
 #include "core/siril.h"
 #include "core/proto.h"
@@ -43,6 +40,8 @@
 #include "gui/histogram.h"	// update_gfit_histogram_if_needed();
 #include "io/ser.h"
 
+#undef STACK_DEBUG
+
 static struct stacking_args stackparam = {	// parameters passed to stacking
 		NULL, NULL, NULL, -1.0, 0, NULL, { '\0' }, NULL, FALSE, { 0, 0 }, -1, 0, { 0, 0 }, NO_REJEC, NO_NORM, FALSE
 };
@@ -71,44 +70,45 @@ void initialize_stacking_methods() {
 	gtk_combo_box_set_active(GTK_COMBO_BOX(rejectioncombo), com.stack.rej_method);
 }
 
-
+/* scale0, mul0 and offset0 are output arguments when i = 0, input arguments otherwise */
 static int _compute_normalization_for_image(struct stacking_args *args, int i,
-		double *offset, double *mul, double *scale, int mode, double *scale0,
+		double *offset, double *mul, double *scale, normalization mode, double *scale0,
 		double *mul0, double *offset0) {
 	imstats *stat = NULL;
 
-	if (args->force_norm || !(stat = seq_get_imstats(args->seq, args->image_indices[i], NULL))) {
+	if (!(stat = seq_get_imstats(args->seq, args->image_indices[i], NULL, STATS_EXTRA))) {
 		fits fit;
 		memset(&fit, 0, sizeof(fits));
 		if (seq_read_frame(args->seq, args->image_indices[i], &fit)) {
 			return 1;
 		}
-		stat = seq_get_imstats(args->seq, args->image_indices[i], &fit);
-		clearfits(&fit);
+		stat = seq_get_imstats(args->seq, args->image_indices[i], &fit, STATS_EXTRA);
+		if (args->seq->type != SEQ_INTERNAL)
+			clearfits(&fit);
 	}
 
 	switch (mode) {
 	default:
 	case ADDITIVE_SCALING:
-		scale[i] = stat->avgdev;
+		scale[i] = stat->scale;
 		if (i == 0)
 			*scale0 = scale[0];
 		scale[i] = *scale0 / scale[i];
 		/* no break */
 	case ADDITIVE:
-		offset[i] = stat->median;
+		offset[i] = stat->location;
 		if (i == 0)
 			*offset0 = offset[0];
 		offset[i] = scale[i] * offset[i] - *offset0;
 		break;
 	case MULTIPLICATIVE_SCALING:
-		scale[i] = stat->avgdev;
+		scale[i] = stat->scale;
 		if (i == 0)
 			*scale0 = scale[0];
 		scale[i] = *scale0 / scale[i];
 		/* no break */
 	case MULTIPLICATIVE:
-		mul[i] = stat->median;
+		mul[i] = stat->location;
 		if (i == 0)
 			*mul0 = mul[0];
 		mul[i] = *mul0 / mul[i];
@@ -117,8 +117,8 @@ static int _compute_normalization_for_image(struct stacking_args *args, int i,
 	return 0;
 }
 
-static int compute_normalization(struct stacking_args *args, norm_coeff *coeff, int mode) {
-	int i, retval = 0, cur_nb = 0;
+int compute_normalization(struct stacking_args *args, norm_coeff *coeff, normalization mode) {
+	int i, retval = 0, cur_nb = 1;
 	double scale0, mul0, offset0;
 	char *tmpmsg;
 
@@ -135,7 +135,17 @@ static int compute_normalization(struct stacking_args *args, norm_coeff *coeff,
 
 	tmpmsg = siril_log_message("Computing normalization...\n");
 	tmpmsg[strlen(tmpmsg) - 1] = '\0';
-	set_progress_bar_data(tmpmsg, PROGRESS_NONE);
+	set_progress_bar_data(tmpmsg, PROGRESS_RESET);
+
+	/* We empty the cache if needed (force to recompute) */
+	if (args->force_norm) {
+		for (i = 0; i < args->seq->number; i++) {
+			if (args->seq->imgparam && args->seq->imgparam[i].stats) {
+				free(args->seq->imgparam[i].stats);
+				args->seq->imgparam[i].stats = NULL;
+			}
+		}
+	}
 
 	// compute for the first image to have scale0 mul0 and offset0
 	if (_compute_normalization_for_image(args, 0, coeff->offset, coeff->mul, coeff->scale, mode,
@@ -144,6 +154,8 @@ static int compute_normalization(struct stacking_args *args, norm_coeff *coeff,
 		return 1;
 	}
 
+	set_progress_bar_data(NULL, 1.0 / (double)args->nb_images_to_stack);
+
 #pragma omp parallel for num_threads(com.max_thread) private(i) schedule(static) if (args->seq->type == SEQ_SER || fits_is_reentrant())
 	for (i = 1; i < args->nb_images_to_stack; ++i) {
 		if (!retval) {
@@ -159,7 +171,7 @@ static int compute_normalization(struct stacking_args *args, norm_coeff *coeff,
 #pragma omp atomic
 			cur_nb++;	// only used for progress bar
 			set_progress_bar_data(NULL,
-					(double) cur_nb / ((double) args->nb_images_to_stack + 1.));
+					(double)cur_nb / ((double)args->nb_images_to_stack));
 		}
 	}
 	set_progress_bar_data(NULL, PROGRESS_DONE);
@@ -186,7 +198,7 @@ int stack_summing(struct stacking_args *args) {
 
 	/* should be pre-computed to display it in the stacking tab */
 	nb_frames = args->nb_images_to_stack;
-	reglayer = get_registration_layer(args->seq);
+	reglayer = get_registration_layer();
 
 	if (nb_frames <= 1) {
 		siril_log_message("No frame selected for stacking (select at least 2). Aborting.\n");
@@ -210,9 +222,10 @@ int stack_summing(struct stacking_args *args) {
 			retval = -1;
 			goto free_and_reset_progress_bar;
 		}
-		tmpmsg = siril_log_message("Processing image %s\n", filename);
-		tmpmsg[strlen(tmpmsg)-1] = '\0';
+		tmpmsg = strdup("Processing image ");
+		tmpmsg = str_append(&tmpmsg, filename);
 		set_progress_bar_data(tmpmsg, (double)cur_nb/((double)nb_frames+1.));
+		free(tmpmsg);
 
 		cur_nb++;	// only used for progress bar
 
@@ -222,6 +235,14 @@ int stack_summing(struct stacking_args *args) {
 			goto free_and_reset_progress_bar;
 		}
 
+		if (args->seq->nb_layers == -1) {
+			/* sequence has not been opened before, this is set in set_seq.
+			 * It happens with the stackall command that stacks a
+			 * sequence right after readseqfile.
+			 */
+			args->seq->rx = fit->rx; args->seq->ry = fit->ry;
+			args->seq->nb_layers = fit->naxes[2];
+		}
 		assert(args->seq->nb_layers == 1 || args->seq->nb_layers == 3);
 		assert(fit->naxes[2] == args->seq->nb_layers);
 
@@ -256,7 +277,9 @@ int stack_summing(struct stacking_args *args) {
 			shiftx = 0;
 			shifty = 0;
 		}
-		siril_log_message("Stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#ifdef STACK_DEBUG
+		printf("Stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#endif
 
 		/* Summing the exposure */
 		exposure += fit->exposure;
@@ -450,12 +473,12 @@ int stack_median(struct stacking_args *args) {
 		naxes[0] = args->seq->ser_file->image_width;
 		naxes[1] = args->seq->ser_file->image_height;
 		ser_color type_ser = args->seq->ser_file->color_id;
-		if (com.raw_set.ser_cfa && type_ser != RGB && type_ser != BGR)
+		if (!com.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR)
 			type_ser = SER_MONO;
 		naxes[2] = type_ser == SER_MONO ? 1 : 3;
 		naxis = type_ser == SER_MONO ? 2 : 3;
 		/* case of Super Pixel not handled yet */
-		if (com.raw_set.bayer_inter == BAYER_SUPER_PIXEL) {
+		if (com.debayer.bayer_inter == BAYER_SUPER_PIXEL) {
 			siril_log_message("Super-pixel is not handled yet for on the fly SER stacking\n");
 			retval = -1;
 			goto free_and_close;
@@ -509,7 +532,7 @@ int stack_median(struct stacking_args *args) {
 	/* Define some useful constants */
 	double total = (double)(naxes[2] * naxes[1] + 2);	// only used for progress bar
 
-	int nb_threads = 1;
+	int nb_threads;
 #ifdef _OPENMP
 	nb_threads = com.max_thread;
 	if (args->seq->type == SEQ_REGULAR && fits_is_reentrant()) {
@@ -522,6 +545,8 @@ int stack_median(struct stacking_args *args) {
 				" stacking will be executed on only one core\n");
 		siril_log_message("Your version of cfitsio does not support multi-threading\n");
 	}
+#else
+	nb_threads = 1;
 #endif
 
 	int nb_channels = naxes[2];
@@ -620,7 +645,7 @@ int stack_median(struct stacking_args *args) {
 	assert(pool_size > 0);
 #endif
 	npixels_in_block = largest_block_height * naxes[0];
-	fprintf(stdout, "allocating data for %d threads (each %'ld MB)\n", pool_size,
+	fprintf(stdout, "allocating data for %d threads (each %'zu MB)\n", pool_size,
 			nb_frames * npixels_in_block * sizeof(WORD) / 1048576L);
 	data_pool = malloc(pool_size * sizeof(struct _data_block));
 	for (i = 0; i < pool_size; i++) {
@@ -797,7 +822,7 @@ int stack_addmax(struct stacking_args *args) {
 
 	/* should be pre-computed to display it in the stacking tab */
 	nb_frames = args->nb_images_to_stack;
-	reglayer = get_registration_layer(args->seq);
+	reglayer = get_registration_layer();
 
 	if (nb_frames <= 1) {
 		siril_log_message("No frame selected for stacking (select at least 2). Aborting.\n");
@@ -821,9 +846,13 @@ int stack_addmax(struct stacking_args *args) {
 			retval = -1;
 			goto free_and_reset_progress_bar;
 		}
-		tmpmsg = siril_log_message("Processing image %s\n", filename);
-		tmpmsg[strlen(tmpmsg)-1] = '\0';
+		tmpmsg = strdup("Processing image ");
+		tmpmsg = str_append(&tmpmsg, filename);
+//		tmpmsg = siril_log_message("Processing image %s\n", filename);
+//		tmpmsg[strlen(tmpmsg)-1] = '\0';
 		set_progress_bar_data(tmpmsg, (double)cur_nb/((double)nb_frames+1.));
+		free(tmpmsg);
+
 		cur_nb++;	// only used for progress bar
 
 		if (seq_read_frame(args->seq, j, fit)) {
@@ -866,7 +895,9 @@ int stack_addmax(struct stacking_args *args) {
 			shiftx = 0;
 			shifty = 0;
 		}
-		siril_log_message("stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#ifdef STACK_DEBUG
+		printf("stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#endif
 
 		/* Summing the exposure */
 		exposure += fit->exposure;
@@ -952,7 +983,7 @@ int stack_addmin(struct stacking_args *args) {
 
 	/* should be pre-computed to display it in the stacking tab */
 	nb_frames = args->nb_images_to_stack;
-	reglayer = get_registration_layer(args->seq);
+	reglayer = get_registration_layer();
 
 	if (nb_frames <= 1) {
 		siril_log_message("No frame selected for stacking (select at least 2). Aborting.\n");
@@ -976,9 +1007,13 @@ int stack_addmin(struct stacking_args *args) {
 			retval = -1;
 			goto free_and_reset_progress_bar;
 		}
-		tmpmsg = siril_log_message("Processing image %s\n", filename);
-		tmpmsg[strlen(tmpmsg)-1] = '\0';
+		tmpmsg = strdup("Processing image ");
+		tmpmsg = str_append(&tmpmsg, filename);
+//		tmpmsg = siril_log_message("Processing image %s\n", filename);
+//		tmpmsg[strlen(tmpmsg)-1] = '\0';
 		set_progress_bar_data(tmpmsg, (double)cur_nb/((double)nb_frames+1.));
+		free(tmpmsg);
+
 		cur_nb++;	// only used for progress bar
 
 		if (seq_read_frame(args->seq, j, fit)) {
@@ -1022,7 +1057,9 @@ int stack_addmin(struct stacking_args *args) {
 			shiftx = 0;
 			shifty = 0;
 		}
-		siril_log_message("stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#ifdef STACK_DEBUG
+		printf("stack image %d with shift x=%d y=%d\n", j, shiftx, shifty);
+#endif
 
 		/* Summing the exposure */
 		exposure += fit->exposure;
@@ -1166,7 +1203,7 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 	norm_coeff coeff;
 
 	nb_frames = args->nb_images_to_stack;
-	reglayer = get_registration_layer(args->seq);
+	reglayer = get_registration_layer();
 
 	if (args->seq->type != SEQ_REGULAR && args->seq->type != SEQ_SER) {
 		char *msg = siril_log_message("Rejection stacking is only supported for FITS images and SER sequences.\nUse \"Sum Stacking\" instead.\n");
@@ -1272,12 +1309,12 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 		naxes[0] = args->seq->ser_file->image_width;
 		naxes[1] = args->seq->ser_file->image_height;
 		ser_color type_ser = args->seq->ser_file->color_id;
-		if (com.raw_set.ser_cfa && type_ser != RGB && type_ser != BGR)
+		if (!com.debayer.open_debayer && type_ser != SER_RGB && type_ser != SER_BGR)
 			type_ser = SER_MONO;
 		naxes[2] = type_ser == SER_MONO ? 1 : 3;
 		naxis = type_ser == SER_MONO ? 2 : 3;
 		/* case of Super Pixel not handled yet */
-		if (com.raw_set.bayer_inter == BAYER_SUPER_PIXEL) {
+		if (com.debayer.bayer_inter == BAYER_SUPER_PIXEL) {
 			siril_log_message("Super-pixel is not handled yet for on the fly SER stacking\n");
 			retval = -1;
 			goto free_and_close;
@@ -1331,7 +1368,7 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 	/* Define some useful constants */
 	double total = (double)(naxes[2] * naxes[1] + 2);	// only used for progress bar
 
-	int nb_threads = 1;
+	int nb_threads;
 #ifdef _OPENMP
 	nb_threads = com.max_thread;
 	if (args->seq->type == SEQ_REGULAR && fits_is_reentrant()) {
@@ -1344,6 +1381,8 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 				" stacking will be executed on only one core\n");
 		siril_log_message("Your version of cfitsio does not support multi-threading\n");
 	}
+#else
+	nb_threads = 1;
 #endif
 
 	int nb_channels = naxes[2];
@@ -1442,7 +1481,7 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 	assert(pool_size > 0);
 #endif
 	npixels_in_block = largest_block_height * naxes[0];
-	fprintf(stdout, "allocating data for %d threads (each %'ld MB)\n", pool_size,
+	fprintf(stdout, "allocating data for %d threads (each %'zu MB)\n", pool_size,
 			nb_frames * npixels_in_block * sizeof(WORD) / 1048576L);
 	data_pool = malloc(pool_size * sizeof(struct _data_block));
 	for (i = 0; i < pool_size; i++) {
@@ -1695,14 +1734,14 @@ int stack_mean_with_rejection(struct stacking_args *args) {
 				case LINEARFIT:
 					do {
 						double *xf = malloc(N * sizeof(double));
-						double *yf = malloc(N*sizeof(double));
+						double *yf = malloc(N * sizeof(double));
 						double a, b, cov00, cov01, cov11, sumsq;
 						quicksort_s(data->stack, N);
-						for (frame=0; frame < N; frame++) {
-							xf[frame] = (double)frame;
-							yf[frame] = (double)data->stack[frame];
+						for (frame = 0; frame < N; frame++) {
+							xf[frame] = (double) frame;
+							yf[frame] = (double) data->stack[frame];
 						}
-						gsl_fit_linear (xf, 1, yf, 1, N, &b, &a, &cov00, &cov01, &cov11, &sumsq);
+						gsl_fit_linear(xf, 1, yf, 1, N, &b, &a, &cov00, &cov01, &cov11, &sumsq);
 						sigma = 0.0;
 						for (frame=0; frame < N; frame++)
 							sigma += (fabs((double)data->stack[frame] - (a*(double)frame + b)));
@@ -1869,23 +1908,27 @@ static void _show_summary(struct stacking_args *args) {
 
 	siril_log_message("Integration of %d images:\n", args->nb_images_to_stack);
 
-	switch (args->normalize) {
-	default:
-	case NO_NORM:
+	if (args->method != &stack_mean_with_rejection) {
 		norm_str = "None";
-		break;
-	case ADDITIVE:
-		norm_str = "additive";
-		break;
-	case MULTIPLICATIVE:
-		norm_str = "multiplicative";
-		break;
-	case ADDITIVE_SCALING:
-		norm_str = "additive + scaling";
-		break;
-	case MULTIPLICATIVE_SCALING:
-		norm_str = "multiplicative + scaling";
-		break;
+	} else {
+		switch (args->normalize) {
+		default:
+		case NO_NORM:
+			norm_str = "None";
+			break;
+		case ADDITIVE:
+			norm_str = "additive";
+			break;
+		case MULTIPLICATIVE:
+			norm_str = "multiplicative";
+			break;
+		case ADDITIVE_SCALING:
+			norm_str = "additive + scaling";
+			break;
+		case MULTIPLICATIVE_SCALING:
+			norm_str = "multiplicative + scaling";
+			break;
+		}
 	}
 
 	siril_log_message("Normalization ............. %s\n", norm_str);
@@ -1956,6 +1999,7 @@ static gboolean end_stacking(gpointer p) {
 		com.uniq->nb_layers = gfit.naxes[2];
 		com.uniq->layers = calloc(com.uniq->nb_layers, sizeof(layer_info));
 		com.uniq->fit = &gfit;
+		com.uniq->fit->maxi = 0;	// force to recompute min/max
 		/* Giving summary if average rejection stacking */
 		_show_summary(args);
 		/* Giving noise estimation */
@@ -2094,7 +2138,7 @@ int stack_filter_included(sequence *seq, int nb_img, double any) {
 int stack_filter_fwhm(sequence *seq, int nb_img, double max_fwhm) {
 	int layer;
 	if (!seq->regparam) return 0;
-	layer = get_registration_layer(seq);
+	layer = get_registration_layer();
 	if (layer == -1) return 0;
 	if (!seq->regparam[layer]) return 0;
 	if (seq->imgparam[nb_img].incl && seq->regparam[layer][nb_img].fwhm > 0.0f)
@@ -2106,7 +2150,7 @@ int stack_filter_fwhm(sequence *seq, int nb_img, double max_fwhm) {
 int stack_filter_quality(sequence *seq, int nb_img, double max_quality) {
 	int layer;
 	if (!seq->regparam) return 0;
-	layer = get_registration_layer(seq);
+	layer = get_registration_layer();
 	if (layer == -1) return 0;
 	if (!seq->regparam[layer]) return 0;
 	if (seq->imgparam[nb_img].incl && seq->regparam[layer][nb_img].quality > 0.0)
@@ -2127,18 +2171,19 @@ int compute_nb_filtered_images() {
 	return count;
 }
 
-/* fill the image_indices mapping in a pre-allocated array */
-void fill_list_of_unfiltered_images(int *indices) {
-	int i, j = 0;
-	if (!sequence_is_loaded()) return;
-	for (i=0, j=0; i<com.seq.number; i++) {
-		if (stackparam.filtering_criterion(
+/* fill the image_indices mapping for the args->image_indices array, which has
+ * to be already allocated to the correct size at least */
+void fill_list_of_unfiltered_images(struct stacking_args *args) {
+	int i, j;
+	for (i=0, j=0; i<args->seq->number; i++) {
+		if (args->filtering_criterion(
 					&com.seq, i,
-					stackparam.filtering_parameter)) {
-			indices[j] = i;
+					args->filtering_parameter)) {
+			args->image_indices[j] = i;
 			j++;
 		}
 	}
+	assert(j <= args->nb_images_to_stack);
 }
 
 /****************************************************************/
@@ -2150,7 +2195,7 @@ double compute_highest_accepted_fwhm(double percent) {
 	int i, layer;
 	double *val = malloc(com.seq.number * sizeof(double));
 	double highest_accepted;
-	layer = get_registration_layer(&com.seq);
+	layer = get_registration_layer();
 	if (layer == -1 || !com.seq.regparam || !com.seq.regparam[layer]) {
 		free(val);
 		return 0.0;
@@ -2185,7 +2230,7 @@ double compute_highest_accepted_quality(double percent) {
 	int i, layer;
 	double *val = malloc(com.seq.number * sizeof(double));
 	double highest_accepted;
-	layer = get_registration_layer(&com.seq);
+	layer = get_registration_layer();
 	if (layer == -1 || !com.seq.regparam || !com.seq.regparam[layer]) {
 		free(val);
 		return 0.0;
@@ -2231,6 +2276,8 @@ void update_stack_interface() {	// was adjuststackspin
 		widgetnormalize = lookup_widget("combonormalize");
 
 	}
+	if (!sequence_is_loaded()) return;
+	stackparam.seq = &com.seq;
 
 	switch (gtk_combo_box_get_active(method_combo)) {
 		default:
@@ -2303,7 +2350,7 @@ void update_stack_interface() {	// was adjuststackspin
 	if (stackparam.nb_images_to_stack >= 2) {
 		if (stackparam.image_indices) free(stackparam.image_indices);
 		stackparam.image_indices = malloc(stackparam.nb_images_to_stack * sizeof(int));
-		fill_list_of_unfiltered_images(stackparam.image_indices);
+		fill_list_of_unfiltered_images(&stackparam);
 		gtk_widget_set_sensitive(go_stack, TRUE);
 	} else {
 		gtk_widget_set_sensitive(go_stack, FALSE);
